/*****************************************************************************
 *		 Generation					             *
 *****************************************************************************/

#include "defs.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/****************************************************************************
 *			 Mutation (integer)				    *
 ****************************************************************************/
int mutation(allele value, double pMut) {
  int tmp;
  if (randomNumber(0.0, 1.0) < pMut) 
    tmp = (value == 0? 1 : 0);  
  else tmp = value;

  return tmp;
}

/****************************************************************************
 *		 Crossover - 2 point 					    *
 ****************************************************************************/
void crossover(allele *parent1, allele *parent2, 
               allele *child1, allele *child2) {
  int tmp, gene, xoverpoint_1=0, xoverpoint_2=0;

  if (flag_no_crossover == 1) {
    xoverpoint_1 = 0;
    xoverpoint_2 = 0;
  } else {
    if (randomNumber(0.0, 1.0) < pCrossover) {
      xoverpoint_1 = 1 + randomInteger(0, lchrom);
      xoverpoint_2 = 1 + randomInteger(0, lchrom);
      while (xoverpoint_1 == xoverpoint_2)
        xoverpoint_2 = 1 + randomInteger(0, lchrom);

      if (xoverpoint_1 > xoverpoint_2) {
	tmp = xoverpoint_1;
	xoverpoint_1 = xoverpoint_2;
	xoverpoint_2 = tmp;
      }
    }
  }
								 
  /* generate individuals */
  for (gene=0; gene<lchrom; gene++) {
    if (gene < xoverpoint_1 || gene >= xoverpoint_2) {
      if (flag_no_mutation == 1) {
	child1[gene] = parent1[gene];  /* child 1 */
	child2[gene] = parent2[gene];				
      }
      else {				
	child1[gene] = mutation(parent1[gene],pMutation); /* child 1 */
	child2[gene] = mutation(parent2[gene],pMutation);
      }
    }
    else {
      if (flag_no_mutation == 1) {
	child1[gene] = parent2[gene];  /* child 1 */
	child2[gene] = parent1[gene];
      }
      else {
	child1[gene] = mutation(parent2[gene],pMutation); /* child 1 */
	child2[gene] = mutation(parent1[gene],pMutation);		
      }
    }
  }	
}

void generation(void){
  int j=0, parent1, parent2;

  // Replace part of the population: RIs
  if (gaMode > 1) substitution( &popcurr ); 

  do {
    // Select two Parents
    parent1 = selection(&popcurr, j);
    parent2 = selection(&popcurr, j+1); 

    // Reproduction
    crossover(popcurr.ind[parent1].chromosome, 
              popcurr.ind[parent2].chromosome, 
              popnew.ind[j].chromosome, 
              popnew.ind[j+1].chromosome);

    // Child j
    popnew.ind[j].fitness = calcFitness(popnew.ind[j].chromosome);
    popnew.ind[j].parent1 = parent1;	 // parent 1 of child j
    popnew.ind[j].parent2 = parent2;	 // parent 2 of child j

    // Child j + 1
    popnew.ind[j+1].fitness = calcFitness(popnew.ind[j+1].chromosome);
    popnew.ind[j+1].parent1 = parent2;  // parent 1 of child j+1
    popnew.ind[j+1].parent2 = parent1;  // parent 2 of child j+1

    j = j + 2;	// increment the index of individual
  } while (j < popsize);
}
