/****************************************************************************
 *			 Selection					    *
 ****************************************************************************/

#include "defs.h"
#include <stdio.h>
#include <stdlib.h>

/****************************************************************************
 *				Elitism				            *
 ****************************************************************************/
int selection_elitism(population *pop, int j, int index_pattern) {
  int individual_chosen;
  if (j == index_pattern)
    individual_chosen = pop->best_individual;
  else 
    individual_chosen = pop->best_individual_2;
	
  return individual_chosen;
}


/****************************************************************************
 *			Roulette Wheel Selection			    *
 ****************************************************************************/
int selection_roulette(population *pop, double sum_fitness) {
  int j, chosen=0, individual_chosen;
  double sum = 0, randnum;

  // random double number 
  randnum = randomNumber(0.0, 1.0); 

  j = 0;
  while ( !chosen ) {
    sum += pop->ind[j].fitness/sum_fitness;
    if (sum >= randnum || j == popsize-1) {
      chosen = 1;
      individual_chosen = j;
    }
    j++;		
  }

  return individual_chosen;
}


/***************************************************************************
 *				Tournament Selection            	   *
 ***************************************************************************/
int selection_tournament(population *pop) {
  int choice1, choice2, individual_chosen;
  double randnum;

  randnum = randomNumber(0.0, 1.0);
  choice1 = randomInteger(0, popsize);
  choice2 = randomInteger(0, popsize);
  if (pop->ind[choice1].fitness > pop->ind[choice2].fitness) {
    if (randnum < tournament_param) 
      individual_chosen = choice1;
    else 
      individual_chosen = choice2;
  }
  else if (pop->ind[choice1].fitness < pop->ind[choice2].fitness) {
    if (randnum < tournament_param) 
      individual_chosen = choice2;
    else 
      individual_chosen = choice1;
  }
  else {
    if (randnum < 0.5) individual_chosen = choice1;
    else individual_chosen = choice2;
  }

  return individual_chosen;
}


/****************************************************************************
 *					Selection			    *
 ****************************************************************************/
int selection(population *pop, int j) {
  int individual_chosen;

  // Standard Selection
  if (elitism == 1 && j<2) {
    individual_chosen = selection_elitism(pop, j, 0);
    flag_no_crossover = 1; // disable crossover
    flag_no_mutation = 1;  // disable mutation
  }
  else {
    individual_chosen = selection_tournament( pop );
    flag_no_crossover= 0; // enable crossover	
    flag_no_mutation = 0; // enable mutation
  }
 
  return individual_chosen;
}
